-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
--Mandatory controller parameters
M.type = "main"
M.relevantDevice = nil
M.engineInfo = {}
M.fireEngineTemperature = 0
M.throttle = 0
M.brake = 0
M.clutchRatio = 0

local min = math.min
local max = math.max
local abs = math.abs

local avToRPM = 9.549296596425384

local motorTop = nil
local motorBottom = nil
local targetRPMRatio = 0
local invMotorTopMaxAV = 0
local invMotorBottomMaxAV = 0
local maxRPM = 0
local rollerTop = nil
local rollerBottom = nil

local function updateGFX(dt)
  targetRPMRatio = min(max(targetRPMRatio + (electrics.values.targetRPMRatioIncrease * dt * 0.5) - (electrics.values.targetRPMRatioDecrease * dt * 0.5), 0), 1)

  local motorTopAV = abs(motorTop.outputAV1)
  local motorBottomAV = abs(motorBottom.outputAV1)
  local rpmRatioTop = min(max(motorTopAV * invMotorTopMaxAV, 0), 1)
  local rpmRatioBottom = min(max(motorBottomAV * invMotorBottomMaxAV, 0), 1)
  local isWrongDirectionTop = motorTop.outputAV1 * motorTop.motorDirection < 0
  local isWrongDirectionBottom = motorBottom.outputAV1 * motorBottom.motorDirection < 0

  local rpmErrorTop = targetRPMRatio - rpmRatioTop
  local rpmErrorBottom = targetRPMRatio - rpmRatioBottom

  local throttleTop = isWrongDirectionTop and 0 or min(max(rpmErrorTop * 10, 0), 1)
  local throttleBottom = isWrongDirectionBottom and 0 or min(max(rpmErrorBottom * 10, 0), 1)

  local brakeTop = isWrongDirectionTop and 1 or min(max(rpmErrorTop * -10, 0), 1)
  local brakeBottom = isWrongDirectionBottom and 1 or min(max(rpmErrorBottom * -10, 0), 1)
  if targetRPMRatio == 0 then
    if rpmErrorTop < 0.05 then
      brakeTop = 1
      throttleTop = 0
    end
    if rpmErrorBottom < 0.05 then
      brakeBottom = 1
      throttleBottom = 0
    end
  end

  rollerTop.desiredBrakingTorque = brakeTop * rollerTop.brakeTorque
  rollerBottom.desiredBrakingTorque = brakeBottom * rollerBottom.brakeTorque
  electrics.values.throttleTop = throttleTop
  electrics.values.throttleBottom = throttleBottom

  electrics.values.throttle = (throttleTop + throttleBottom) * 0.5
  electrics.values.brake = (brakeTop + brakeBottom) * 0.5
  electrics.values.clutch = 0
  electrics.values.clutchRatio = 1
  electrics.values.gear = string.format("%d%%", targetRPMRatio * 100)
  electrics.values.gearIndex = 0
  electrics.values.rpm = (motorTopAV + motorBottomAV) * 0.5 * avToRPM
  electrics.values.oiltemp = 0
  electrics.values.watertemp = 0

  M.engineInfo = {
    electrics.values.rpm
    , maxRPM
    , 0
    , 0
    , electrics.values.rpm
    , electrics.values.gear
    , 0
    , 0
    , 0
    , 0
    , obj:getVelocity():length()  -- airspeed
    , 0
    , 0
    --, v.data.engine.transmissionType
    , "manual"
    , obj:getID()
    , 0
    , 0
    , 1
  }
end

local function toggleDirection()
  motorTop.motorDirection = motorTop.motorDirection * -1
  motorBottom.motorDirection = motorBottom.motorDirection * -1
end

local function init(jbeamData)
  motorTop = powertrain.getDevice("motorTop")
  motorBottom = powertrain.getDevice("motorBottom")
  invMotorTopMaxAV = 1 / motorTop.maxAV
  invMotorBottomMaxAV = 1 / motorBottom.maxAV
  maxRPM = 2000
  targetRPMRatio = 0
  electrics.values.targetRPMRatioIncrease = 0
  electrics.values.targetRPMRatioDecrease = 0

  for k,v in pairs(wheels.wheelRotators) do
    if v.name == "motor_FL" then rollerTop = wheels.wheelRotators[k] end
    if v.name == "motor_FR" then rollerBottom = wheels.wheelRotators[k] end
  end
end

local function sendTorqueData()
  if not playerInfo.firstPlayerSeated then return end
  if motorTop then
    motorTop:sendTorqueData()
  end
  if motorBottom then
    motorBottom:sendTorqueData()
  end
end

M.init = init
M.updateGFX = updateGFX

--Mandatory main controller API
M.shiftUp = nop
M.shiftDown = nop
M.shiftToGearIndex = nop
M.cycleGearboxModes = nop
M.setGearboxMode = nop
M.setStarter = nop
M.setFreeze = nop
M.sendTorqueData = sendTorqueData
-------------------------------
M.toggleDirection = toggleDirection

return M